//
//  File:         sensor_i.h
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 02.10.97
//
// General Description:
//   Header file for the implementation class of the Sensor interface
//

#ifndef sensor_i_h
#define sensor_i_h

#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>
#include <errno.h>
#include <string.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#include "abng_core_management.hh"
#include "abng_guarded.hh"
#include "abng_string.h"
#include "compatibility.hh"

class Sensor_I
{
 private:
  // state of the IDL Sensor object
  ABng_Core_System::Sensor::Description my_descr;

  // reference of CORBA object encapsulated this C++ object
  ABng_Core_System::Sensor_ptr my_corba_object;

  // state lock
  ABng_Guarded &my_guard;

 public:
  // ctor
   Sensor_I (
	const ABng_Core_System::Sensor::Description& descr_init
   );

#ifdef FilePersistency
   Sensor_I (
	istream &iostr
   );
#endif

  virtual 
   ~Sensor_I ();

   operator==(	
	const Sensor_I&
   );

  // methods enabling access to the state

  void
   set_descr (
	const ABng_Core_System::Sensor::Description& descr_update
   );
 
  const ABng_Core_System::Sensor::Description&
   get_descr () const;

  void
   set_corba_object (
	const ABng_Core_System::Sensor_ptr corba_object_p
   );

  const ABng_Core_System::Sensor_ptr
   get_corba_object ();

  void 
   set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
   );

  //
  // IDL operations of the System_Static_Data_Observed_E interface
  //                       _____________________________

  virtual ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
   system_static_data_observer_list (
	ENV_DECL1(env)
   )
		throw (CORBA::SystemException); 
                
  virtual void 
   attach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs 
	ENV_DECL(env)
   )
		throw (CORBA::SystemException);

  virtual void 
   detach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer); 

  //
  // IDL operations of the Scheduler_Request_Queue interface
  //                       _______________________

  virtual ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests*
   list_user_requests (
	const char * user_id
	ENV_DECL(env)
   )
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable);

          
  virtual ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests* 
   list_all_requests (
	ENV_DECL1(env)
   )
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable);

  virtual void 
   remove_request (
	CORBA::Long request_id
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       ABng_Core::Unknown_Request_Id,
		       Domain_Repository::Destination_Unreachable);
	
  //
  // IDL operations of the Sensor_Commander interface
  //                       ________________

  // auxiliary method for Sensor_Commander interface
  virtual void 
   prepare_scheduler_params ( 
	const ABng_Core_System::Badges& badges,
	ABng_Core_System::Badge_Ids &bids,
	ABng_Core_System::Sensor_Ids &sids
   ) 
      		throw (ABng_Core::Invalid_Argument); 

  void
   map_exceptions( 
	CORBA::Exception &ex
  ) 
      		throw (ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   issue_sensor_command (
	const ABng_Core_System::Octets& cmd, 
	const CORBA::Boolean has_reply,
	const CORBA::Octet reply_code,
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to 
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   issue_badge_command (
	const ABng_Core_System::Badges& badges, 
	const ABng_Core_System::Octets& cmd, 
	const CORBA::Boolean has_reply,
	const CORBA::Octet reply_code,
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   challenge (
	const ABng_Core_System::Badges& badges,
	const ABng_Core_System::Octets& random, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to 
	ENV_DECL(env)
  ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   sound (
	const ABng_Core_System::Badges& badges, 
	const ABng_Core_System::Octets& sound, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to 
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   set_state (
	const ABng_Core_System::Badges& badges,
	CORBA::Octet status,
	const ABng_Core_System::Octets& sound, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   get_state (
	const ABng_Core_System::Badges& badges, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   set_home (
	const ABng_Core_System::Badges& badges,
	const ABng_Core_System::Octets& address,
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   get_home (
	const ABng_Core_System::Badges& badges,
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   reset_buffer (
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to 
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   reset_fifos (
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   get_station_state (
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   set_output (
	CORBA::Octet output_value, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  virtual CORBA::Long 
   set_persistence (
	CORBA::Boolean mode, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable);

  //
  // IDL operations of the Sensor interface
  //                       ______
 
  virtual ABng_Core_System::Sensor::Description* 
   descr (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual ABng_Core_System::Sensor::Value_Description* 
   value_descr (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual Domain_Repository::Repository_Item_Id 
   item_id (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual ABng_Core_System::Sensor_Id* 
   id (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

#ifdef ObjectStore
  //
  // method for ObjectStore
  //

  static os_typespec* get_os_typespec();

#elif defined FilePersistency
  // methods to store and restore object of this class
  virtual void
   save (
	ostream &iostr
   );

  virtual void
   restore (
	istream &iostr
   );
#endif
};   

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_ABng_Core_System_Sensor(Sensor_I)
#else
DEF_TIE_ABng_Core_System_Sensor(Sensor_I)
#endif

#elif defined OMNIORB
// DEF_TIE__tie_Sensor
#endif

// implementation of this class need method "instance" of class Sensor_Rep_I
#include "sensor_rep_i.h"

#endif /* sensor_i_h */
