#ifdef ORBIX
#define EXCEPTIONS
#endif

#include "dyn_dispatcher.hh"
#include <rw/tvdlist.h>
#include <unistd.h>

//
// Future implementation: 
// - move remote calls of receive_sighting to asynchronous thread
// - discard sighting messages that are invalid because of new sightings
//   that arrived in the meantime
//

DynDispatcher::DynDispatcher (
#ifdef FilePersistency
    const char *repository_name_p,
#endif
    unsigned long _burst_delay
    ) :
  Basic_Observed_I<DDO_Types>(
#ifdef FilePersistency
			      repository_name_p
#endif
			      ),
  burst_delay( _burst_delay),
  buffer_released( &buffer_guard.mutex()),
  buffer_data( &buffer_guard.mutex()),
  buffer_sleep( &buffer_guard.mutex()),
  active_buffer(0)
{
  burst_delay= 500000;
  for (int i=0; i<2; i++)
    burst_buffer[i]= new Burst_Buffer_Type( hash_observer);
  (new omni_thread( static_dyn_worker, this))->start();
}

void DynDispatcher::static_dyn_worker( void *args) {
   ((DynDispatcher*)args)->dyn_worker();
}

void DynDispatcher::dyn_worker() {
  do {

    int buffer_no= get_buffer_to_send();
    //    cerr << "####Buffer to send " << buffer_no << endl;
    unsigned long my_burst_time;
    {
      ABng_S_Guarded::auto_lock local_lock( buffer_guard);
      gettimeofday( &start_time, NULL);
      my_burst_time= burst_delay;
    }
    int dead_observers= 0;
    Burst_Buffer_Iterator iterator(*burst_buffer[buffer_no]);
    while (iterator()) {
      try {
	iterator.key()->receive_sightings( *(iterator.value()));
      }
      catch (...) {
	dead_observers= 1;
	ABng_Guarded::auto_lock local_lock(*this);
	try {
	  private_detach_observer(iterator.key(),
				  get_item_ids_list(iterator.key()));
	} 
	catch(...) {
	}
      }
    }
    Burst_Buffer_Iterator del_iterator(*burst_buffer[buffer_no]);
    while (del_iterator()) {
      CORBA::release( del_iterator.key());
      delete ( del_iterator.value());
    }
    burst_buffer[buffer_no]->clear();

    if (dead_observers) save_observers();
    
    struct timeval stop_time;
    gettimeofday(&stop_time, NULL);
    long delay= (stop_time.tv_sec-start_time.tv_sec)*1000000+(stop_time.tv_usec-start_time.tv_usec);

    if (delay<my_burst_time) {
      ABng_S_Guarded::auto_lock local_lock( buffer_guard);
      unsigned long sec,nsec;
      unsigned long t= my_burst_time-delay;
      //      cerr << "#####sleep for " << t << " us" << endl;
      omni_thread::get_time(&sec,&nsec,
			    t/1000000,(t%1000000)*1000);
#ifdef OMNIORB
      buffer_sleep.timedwait( sec, nsec);
#else
      buffer_sleep.timed_wait( sec, nsec);
#endif
    }
  } while (1);
}

void DynDispatcher::set_burst_delay( unsigned long new_delay) {
  ABng_S_Guarded::auto_lock local_lock( buffer_guard);
  burst_delay= new_delay;
  buffer_sleep.signal();
}

int DynDispatcher::get_buffer_to_send() {
  ABng_S_Guarded::auto_lock local_lock( buffer_guard);
  int ret= active_buffer;
  
  if (! burst_buffer[ret]->entries()) {
    buffer_data.wait();
    unsigned long sec,nsec;
    omni_thread::get_time(&sec,&nsec,
			  burst_delay/1000000,(burst_delay%1000000)*1000);
#ifdef OMNIORB
    buffer_sleep.timedwait( sec, nsec);
#else
    buffer_sleep.timed_wait( sec, nsec);
#endif
  }
  
  active_buffer= 1-active_buffer;
  return ret;
}

void DynDispatcher::add_sighting( Basic_Observed_I<DDO_Types>::Observer_ptr obs,
				  ABng_Core::Sighting_Description &sd) {
  //  cerr << "add_sighting" << endl;
  if (!burst_buffer[active_buffer]->contains(obs)){
    //    cerr << "add_sighting - new entry" << endl;
    // a new entry
    ABng_Core::Sighting_Descriptions *sds=
      new ABng_Core::Sighting_Descriptions(80);
    sds->length(1);
    (*sds)[0]= sd;
    burst_buffer[active_buffer]->insertKeyAndValue(Observer::_duplicate(obs), 
						   sds);
  }
  else {
    //    cerr << "add_sighting - add entry" << endl;
    ABng_Core::Sighting_Descriptions *sds= NULL;
    burst_buffer[active_buffer]->findValue(obs, sds);
    sds->length( sds->length()+1);
    (*sds)[sds->length()-1]= sd;
  }  
}

int DynDispatcher::dynamic_notify_observer(
				Observer_ptr obs,
				ABng_Core::Sighting_Description &sd) {
  if (burst_delay) {
    ABng_S_Guarded::auto_lock local_lock( buffer_guard);
    add_sighting( obs, sd);
    buffer_data.signal();
    return 1;
  }
  else {
    try {
      obs->receive_sighting(sd);
    }
    catch (CORBA::COMM_FAILURE &ex) {
      cerr << "The observer is dead" << endl;
      return 0;
    }
    catch (CORBA::Exception &ex) {
#ifdef DEBUG
      cerr << "Caught during notify call " << ex << endl;
#endif
      return 0;
    }
  }
}

void DynDispatcher::dynamic_notify_observers( 
				Basic_Observed_I<DDO_Types>::Observer_List &obs,
				ABng_Core::Sighting_Description &sd) {

  if (burst_delay) {
    ABng_S_Guarded::auto_lock local_lock( buffer_guard);
    Basic_Observed_I<DDO_Types>::Observer_List_Iterator 
      iterator(obs);

    //    cerr << "dynamic_notify : " << (int) obs.entries() << " entries" << endl;
    // Notify all observers of this entity
    while (iterator())
      add_sighting( iterator.key(), sd);
    if (obs.entries())
      buffer_data.signal();
    return;
  }
  else {
    RWTValDlist<DDO_Types::Observer_ptr> dead_observers;

    // Send a current sighting of the locatable to them.
    Basic_Observed_I<DDO_Types>::Observer_List_Iterator 
      iterator(obs);
    
    // Notify all observers of this entity
    while (iterator()){
      try {
	iterator.key()->receive_sighting( sd);
      }
      catch (CORBA::COMM_FAILURE &ex) {
	cerr << "The observer is dead" << endl;
	dead_observers.insert(iterator.key());
      }
      catch (CORBA::Exception &ex) {
#ifdef DEBUG
	cerr << "Caught during notify call " << ex << endl;
#endif
      }
    }
    RWTValDlistIterator<DDO_Types::Observer_ptr> dead_iterator(dead_observers);
  
    // Remove dead observers.
    while (dead_iterator()) {
      ABng_Guarded::auto_lock local_lock(*this);
      try {
	private_detach_observer(dead_iterator.key(),
				get_item_ids_list(dead_iterator.key()));
      } 
      catch(...) {
      }
    }  
    if (dead_observers.entries()) save_observers();
  }
}


